@@ -1,4 +1,5 @@
 import json
+from json import JSONDecodeError
 import logging
 from cachetools import TTLCache
 from requests import request
@@ -6,9 +7,8 @@ from ad_api.auth.credentials import Credentials
 from ad_api.auth import AccessTokenClient, AccessTokenResponse
 from .api_response import ApiResponse
 from .base_client import BaseClient
-from .exceptions import get_exception_for_content
+from .exceptions import get_exception_for_code, get_exception_for_content
 from .marketplaces import Marketplaces
-import sys
 import os
 import requests
 from io import BytesIO
@@ -16,15 +16,13 @@ import gzip
 from zipfile import ZipFile
 import zipfile
 from urllib.parse import urlparse, quote
+from ad_api.base.credential_provider import CredentialProvider
 
 log = logging.getLogger(__name__)
 role_cache = TTLCache(maxsize=int(os.environ.get('AD_API_AUTH_CACHE_SIZE', 10)), ttl=3200)
 
 
 class Client(BaseClient):
-    access_token_client_class = AccessTokenClient
-    credentials_class = Credentials
-    grantless_scope = ''
 
     def __init__(
             self,
@@ -32,23 +30,32 @@ class Client(BaseClient):
             marketplace: Marketplaces = Marketplaces[os.environ[
                 'AD_API_DEFAULT_MARKETPLACE']] if 'AD_API_DEFAULT_MARKETPLACE' in os.environ else Marketplaces.EU,
             credentials=None,
+            proxies=None,
+            verify=True,
+            timeout=None,
             debug=False
     ):
 
-        super().__init__(account, credentials)
+        self.credentials = CredentialProvider(account, credentials).credentials
+        self._auth = AccessTokenClient(
+            credentials=self.credentials,
+            proxies=proxies,
+            verify=verify,
+            timeout=timeout,
+        )
         self.endpoint = marketplace.endpoint
         self.debug = debug
-        self._auth = self.access_token_client_class(
-            account=account, credentials=credentials, credentials_class=self.credentials_class
-        )
+        self.timeout = timeout
+        self.proxies = proxies
+        self.verify = verify
 
     @property
     def headers(self):
         return {
             'User-Agent': self.user_agent,
-            'Amazon-Advertising-API-ClientId': self.credentials.client_id,
+            'Amazon-Advertising-API-ClientId': self.credentials['client_id'],
             'Authorization': 'Bearer %s' % self.auth.access_token,
-            'Amazon-Advertising-API-Scope': self.credentials.profile_id,
+            'Amazon-Advertising-API-Scope': self.credentials['profile_id'],
             'Content-Type': 'application/json'
         }
 
@@ -62,7 +69,15 @@ class Client(BaseClient):
         location = params.get("url")
 
         try:
-            r = requests.get(location, headers=headers or self.headers, data=None, allow_redirects=True)
+            r = requests.get(
+                location,
+                headers=headers or self.headers,
+                data=None,
+                allow_redirects=True,
+                timeout=self.timeout,
+                proxies=self.proxies,
+                verify=self.verify,
+            )
 
         except requests.exceptions.InvalidSchema as e:
             error = {
@@ -194,7 +209,6 @@ class Client(BaseClient):
             next_token = None
             return ApiResponse(name + ".zip", next_token, headers=r.headers)
 
-
         else:
 
             error = {
@@ -206,8 +220,7 @@ class Client(BaseClient):
             next_token = None
             return ApiResponse(error, next_token, headers=self.headers)
 
-        sys.exit()
-
+        raise NotImplementedError("Unknown mode")
 
     def _request(self,
                  path: str,
@@ -226,38 +239,31 @@ class Client(BaseClient):
             base_header.pop("Content-Type")
             headers = base_header
 
-
         elif headers is not None:
 
             base_header = self.headers.copy()
             base_header.update(headers)
             headers = base_header
 
-        if method in ('POST', 'PUT', 'PATCH'):
-
-            res = request(method,
-                          self.endpoint + path,
-                          params=params,
-                          data=data,
-                          headers=headers or self.headers)
-
-        else:
-            res = request(method,
-                          self.endpoint + path,
-                          params=params,
-                          headers=headers or self.headers)
+        request_data = data if method in ('POST', 'PUT', 'PATCH') else None
+        res = request(
+            method,
+            self.endpoint + path,
+            params=params,
+            data=request_data,
+            headers=headers or self.headers,
+            timeout=self.timeout,
+            proxies=self.proxies,
+            verify=self.verify,
+        )
 
         if self.debug:
             logging.info(headers or self.headers)
 
-            # logging.info(params)
-            # logging.info(type(params))
 
             if params:
                 str_query = ""
                 for key, value in params.items():
-                    # logging.info(key)
-                    # logging.info(value)
                     str_query += key + "=" + quote(str(value))
                 message = method + " " + self.endpoint + path + "?" + str_query
             else:
@@ -267,76 +273,48 @@ class Client(BaseClient):
             if data is not None:
                 logging.info(data)
 
-        return self._check_response(res)
-
-    # @staticmethod
-    def _check_response(self, res) -> ApiResponse:
-
-        if self.debug:
             logging.info(vars(res))
 
+        return self._check_response(res)
+
+    @staticmethod
+    def _check_response(res) -> ApiResponse:
+
         content = vars(res).get('_content')
-        str_content = content.decode('utf8')
-
-        if type(str_content) is str and str_content[0:50] == '<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">' and vars(res).get('_content_consumed') is True:
-
-            dictionary = {"status_code": vars(res).get('status_code'), "msg": "Unauthorized"}
-            exception = get_exception_for_content(dictionary)
-            raise exception(dictionary)
-
-        if type(str_content) is str and str_content[0:15] == 'Invalid request' and vars(res).get('_content_consumed') is True:
-            dictionary = {"status_code": vars(res).get('status_code'), "msg": str_content}
-            exception = get_exception_for_content(dictionary)
-            raise exception(dictionary)
-
-        data = json.loads(str_content)
-
-        if type(data) is dict and data.get('code') == 'UNAUTHORIZED':
-            exception = get_exception_for_content(data)
-            raise exception(data)
-
-        if type(data) is dict and data.get('message') == 'Too Many Requests' and vars(res).get('_content_consumed') is True:
-            exception = get_exception_for_content(data)
-            raise exception(data)
-
-        if type(data) is dict and data.get('code') == 'NOT_FOUND' and vars(res).get('_content_consumed') is True:
-            dictionary = {"status_code": vars(res).get('status_code'), "code": data.get('code'), "details": data.get('details'), "requestId": data.get('requestId')}
-            exception = get_exception_for_content(data)
-            raise exception(dictionary)
-
-        if type(data) is dict and data.get('code') == 'SERVER_IS_BUSY' and vars(res).get('_content_consumed') is True:
-            dictionary = {"status_code": vars(res).get('status_code'), "code": data.get('code'), "details": data.get('details'), "requestId": data.get('requestId')}
-            exception = get_exception_for_content(data)
-            raise exception(dictionary)
-
-        if type(data) is dict and data.get('message') == 'Unauthorized' and vars(res).get('_content_consumed') is True:
-            dictionary = {"status_code": vars(res).get('status_code'), "message": "Unauthorized"}
-            exception = get_exception_for_content(dictionary)
-            raise exception(dictionary)
-
-        if type(data) is dict and data.get('details') == 'Invalid authorization inputs' and vars(res).get('_content_consumed') is True:
-            dictionary = {"status_code": vars(res).get('status_code'), "message": "Invalid authorization inputs"}
-            exception = get_exception_for_content(dictionary)
-            raise exception(dictionary)
-
-        if type(data) is dict and data.get('message') == 'Missing Authentication Token' and vars(res).get('_content_consumed') is True:
-            dictionary = {"status_code": vars(res).get('status_code'), "message": "Missing Authentication Token"}
-            exception = get_exception_for_content(dictionary)
-            raise exception(dictionary)
-
-        if type(data) is dict and data.get('details') == 'Cannot consume content type' and vars(res).get('_content_consumed') is True:
-            dictionary = {"status_code": vars(res).get('status_code'), "message": data.get('details')}
-            exception = get_exception_for_content(dictionary)
-            raise exception(dictionary)
-
-        if vars(res).get('_content') == b'[]' and vars(res).get('_content_consumed') is True:
-            # data = json.loads('{"status_code": 200, "msg": "No Data Available", "payload": "No Data Available"}')
-            data = vars(res).get('_content').decode('utf-8')
-            # dictionary = {"status_code": 200, "message": "No Data Available"}
-            # exception = get_exception_for_content(dictionary)
-            # raise exception(dictionary)
-
         headers = vars(res).get('headers')
         status_code = vars(res).get('status_code')
-        next_token = vars(res).get('_next')
-        return ApiResponse(data, next_token, headers=headers)
+
+        if 200 <= res.status_code < 300:
+
+            try:
+                js = res.json() or {}
+            except JSONDecodeError:
+                js = {}
+
+            try:
+                error = js.get('error', None) # Dict.get(key, default=None)
+            except AttributeError:
+                error = None
+
+            if error:
+                exception = get_exception_for_content(error[0].get('code'))
+                raise exception(error[0].get('code'), error[0], headers)
+
+            next_token = vars(res).get('_next')
+
+            return ApiResponse(js, next_token, headers=headers)
+
+        else:
+
+            exception = get_exception_for_code(res.status_code)
+
+            try:
+                js = res.json()
+            except JSONDecodeError:
+                js = res.content
+
+
+            raise exception(status_code, js, headers)
+            exit(res.status_code)
+
+
